package lc.block;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.files.FileHandle;
import java.util.HashMap;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.json.JSONArray;
import org.json.JSONObject;

/**
 * @author Death
 */
public class Block {

    public static int SIZE = 32;
    public static HashMap<Integer, Block> blocks;
    public static HashMap<String, Integer> ids;
    public static BlocksTexture texture;
    
    public static void load(FileHandle path){
        Block.blocks = new HashMap<>();
        Block.ids = new HashMap<>();
        
        JSONObject ob = new JSONObject(path.readString());
        
        BlocksTexture blTex = new BlocksTexture();
        blTex.setNames(ob.getString("namesTexture"));
        if(ob.getBoolean("pathLocal")){
            blTex.path = Gdx.files.local(ob.getString("pathTexture"));
        }else{
            blTex.path = Gdx.files.internal(ob.getString("pathTexture"));
        }
        blTex.load();
        texture = blTex;
        
        JSONArray blocksAr = ob.getJSONArray("blocks");
        for(int i = 0; i < blocksAr.length(); i++){
            JSONObject blockInfa = blocksAr.getJSONObject(i);
            
            String classPath = blockInfa.getString("classPath");
            if(!classPath.equals("this")){
                try{
                    Block block = (Block)Class.forName(classPath).newInstance();
                    block.init();
                    ids.put(block.name, block.id);
                    blocks.put(block.id, block);
                }
                catch(ClassNotFoundException ex){
                    System.err.println("Error! Class Loader \"" + classPath + "\"");
                    System.exit(0);
                }
                catch(InstantiationException | IllegalAccessException ex){
                    Logger.getLogger(Block.class.getName()).log(Level.SEVERE, null, ex);
                }
                continue;
            }
            
            String name = blockInfa.getString("name");
            int id = blockInfa.getInt("id");
            int rotate = blockInfa.getInt("rotate");
            String nameTex = blockInfa.getString("texture");
            
            Block block = new Block().setId(id).setName(name).setTexture(nameTex).setRotate(rotate);
            block.init();
            ids.put(name, id);
            blocks.put(id, block);
        }
    }
    
    public static Block get(String name){
        return blocks.get(ids.get(name));
    }
    
    public int id;
    public int rotate;
    public String name;
    public String nameTexture;
    
    public Block setRotate(int f){
        this.rotate = f;
        return this;
    }
    
    public Block setId(int i){
        this.id = i;
        return this;
    }
    
    public Block setTexture(String texture){
        this.nameTexture = texture;
        return this;
    }
    
    public Block setName(String name){
        this.name = name;
        return this;
    }
    
    public void init(){};
}
