package lc.world.objectes;

import com.badlogic.gdx.graphics.g2d.Sprite;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.math.Rectangle;
import lc.block.Block;
import lc.world.Ground;
import lc.world.World;

/**
 * @author Death
 */
public class WorldObject {

    public static final int UP = 0, DOWN = 1, LEFT = 2, RIGHT = 3, VERTICALLY = 5, HORIZONTAL = 6;
    
    
    private World world = null;
    private Ground ground = null;
    private final Rectangle rect;
    
    private long timeDelayMove;
    
    public CollisionGround collisionGround = null;
    public CollisionGroundExpose groundExpose = null;
    public boolean onLand;
    
    public float x, y, speedX, speedY;
    public float maxSpeedX, maxSpeedY;
    public float width, height;
    public boolean gravity;
    public boolean[] moveKey;
    public boolean[] moveDir;
    public Sprite sprite = null;
    public TypeObjectWorld type;

    public WorldObject(){
        this.rect = new Rectangle();
        this.type = TypeObjectWorld.STATIC;
        this.moveKey = new boolean[4];
        this.moveDir = new boolean[4];
        this.maxSpeedX = 5;
        this.maxSpeedY = 5;
        this.gravity = false;
        this.onLand = false;
    }
    
    public void draw(SpriteBatch b){
        if(sprite == null) return;
        sprite.setPosition(x, y);
        sprite.setSize(width, height);
        sprite.draw(b);
    }
    
    public Rectangle getRect(){
        return rect.set(x, y, width, height);
    }
    
    public void initPhysics(World map){
        this.initPhysics(map, true);
    }
    
    public void initPhysics(World map, boolean g){
        this.gravity = g;
        this.world = map;
        this.ground = map.ground;
    }
    
    private float acceleration = 5;//Ускорение
    public void move(float f){
        
        if(moveKey[UP]){
            speedY += acceleration * f;
            moveDir[UP] = true;
            if(speedY > maxSpeedY) speedY = maxSpeedY;
        }else{
            if(moveDir[UP]){
                speedY -= acceleration * f;
                if(speedY < 0){
                    speedY = 0;
                    moveDir[UP] = false;
                }
            }
        }
        
        if(moveKey[DOWN]){
            speedY -= acceleration * f;
            if(speedY < -maxSpeedY) speedY = -maxSpeedY;
        }
        
        if(moveKey[RIGHT]){
            speedX += acceleration * f;
            moveDir[RIGHT] = true;
            if(speedX > maxSpeedX) speedX = maxSpeedX;
        }else{
            if(moveDir[RIGHT]){
                speedX -= acceleration * 2 * f;
                if(speedX < 0){
                    speedX = 0;
                    moveDir[RIGHT] = false;
                }
            }
        }
        
        if(moveKey[LEFT]){
            speedX -= acceleration * 2 * f;
            moveDir[LEFT] = true;
            if(speedX < -maxSpeedX) speedX = -maxSpeedX;
        }else{
            if(moveDir[LEFT]){
                speedX += acceleration * f;
                if(speedX > 0){
                    speedX = 0;
                    moveDir[LEFT] = false;
                }
            }
        }
        
        this.gravity(f);
        
        if(moveY(speedY)){
            speedY = 0;
            moveKey[DOWN] = false;
            moveKey[UP] = false;
            moveDir[UP] = false;
            moveDir[DOWN] = false;
        }
        if(moveX(speedX)){
            speedX = 0;
            moveKey[LEFT] = false;
            moveKey[RIGHT] = false;
            moveDir[RIGHT] = false;
            moveDir[LEFT] = false;
        }
    }
    
    private void gravity(float f){
        if(!this.gravity && onLand) return;
        
        speedY -= 4 * f;
        if(speedY < maxSpeedY * 2) speedY = -(maxSpeedY * 2);
    }
    
    public boolean moveX(float speed){
        if(speed == 0) return false;
        if(Math.abs(speed) > Block.SIZE){
            superMoveX(Math.round(speed));
            return true;
        }
        this.x = this.x + speed;
        return collisionWall(Math.round(speed), HORIZONTAL);
    }
    public boolean moveY(float speed){
        if(speed == 0) return false;
        if(Math.abs(speed) > Block.SIZE){
            superMoveY(Math.round(speed));
            return true;
        }
        this.y = this.y + speed;
        return collisionWall(Math.round(speed), VERTICALLY);
    }
    
    private void superMoveX(int speed){
        if(speed != 0){
            int sizeCell = this.getSizeCell();
            int sx = Math.abs(speed);
            int vx = sx / speed;
            while(sx > sizeCell){
                this.x = this.x + Block.SIZE * vx;
                sx -= sizeCell;
                collisionWall(speed, 0);
                if(speed == 0){
                    sx = 0;
                    break;
                }
            }
            this.x = this.x + sx * vx;
            collisionWall(speed, 0);
        }
    }
    
    private void superMoveY(int speed){
        if(speed != 0){
            int sizeCell = this.getSizeCell();
            int sy = Math.abs(speed);
            int vy = sy / speed;
            while(sy > sizeCell){
                this.y = this.y + sizeCell * vy;
                sy -= sizeCell;
                collisionWall(speed,1);
                if(speed == 0){
                    sy = 0;
                    break;
                }
            }
            this.y = this.y + sy * vy;
            collisionWall(speed, 1);
        }
    }
    
    public boolean collisionWall(int speed, int dir){
        if(world == null || ground == null) return false;
        if(collisionBorder()) return true;
        for(int i = (int)(this.x / this.getSizeCell()); i < (float)(this.x + this.width) / this.getSizeCell(); i++){
            for(int j = (int)(this.y / this.getSizeCell()); j < (float)(this.y + this.height) / this.getSizeCell(); j++){
                boolean collision = ground.getBlock(i, j) != null;
                if(collision && groundExpose != null){
                    collision = groundExpose.expose(ground.getBlock(i, j), i, j);
                }
                if(collision){
                    int dirColl = 0;
                    if(dir == HORIZONTAL){
                        if(speed > 0){
                            this.x = i * this.getSizeCell() - this.width;
                            dirColl = RIGHT;
                        }else if(speed < 0){
                            this.x = i * this.getSizeCell() + this.getSizeCell();
                            dirColl = LEFT;
                        }
                    }else if(dir == VERTICALLY){
                        if(speed > 0){
                            this.y = j * this.getSizeCell() - this.height;
                            dirColl = UP;
                        }else if(speed < 0){
                            this.onLand = true;
                            this.y = j * this.getSizeCell() + this.getSizeCell();
                            dirColl = DOWN;
                        }
                    }
                    if(dirColl != 0 && collisionGround != null){
                        collisionGround.collision(i, j, dirColl);
                    }
                    return true;
                }
            }
        }
        return false;
    }
    
    private boolean collisionBorder(){
        if(this.x < 0){
            x = 0;
            return true;
        }
        if(this.y < 0){
            y = 0;
            return true;
        }
        if(this.x + this.width > world.getWidth()){
            this.x = world.getWidth() - this.width;
            return true;
        }
        if(this.y + this.height > world.getHeight()){
            this.y = world.getHeight() - this.height;
            return true;
        }
        return false;
    }
    
    public int getSizeCell(){
        return Block.SIZE;
    }
}
